<?php

namespace modules\catalogs\Http\Controllers;

use App\Images;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use modules\catalogs\Models\Category;

class CategoryController extends Controller
{

    private function pageRoute(String $slug)
    {
        $category = null;
        $paramIndex = [];
        $paramCreate = [];
        $routes = [
            "index" => route('app.catalogs.category.index'),
            "create" => route('app.catalogs.category.create'),
        ];

        $category = Category::where('slug', $slug)->first();
        if ($category) {
            $paramCreate = ['category' => $category->slug];

            if ($category->parent_id != null) {
                $parent = Category::find($category->parent_id);
                if ($parent) {
                    $paramIndex = ['category' => $parent->slug];
                }
            }

            $routes = [
                "index" => route('app.catalogs.category.index', $paramIndex),
                "create" => route('app.catalogs.category.create', $paramCreate),
            ];
        }

        return [
            'category' => $category,
            'route' => $routes,
        ];
    }

    public function index(Request $request){       
        $slug = "";
        if($request->has('category')){
            $slug = $request->category;
        }        

        $response = $this->pageRoute($slug);
        $category = $response['category'];
        $route = $response['route'];

        if($request->ajax()){
            return response()->json([
                'status' => 'success',
                'category' => $category
            ]);
        }
        
        return view("catalogs::category.index", [
            "category" => $category,
            'route' => $route
        ]);
    }

    public function categories(Request $request){
        $page = isset($request->pagesize) ? $request->pagesize : 25;
        $res = Category::with('parent', 'categories');

        if($request->has("category")){
            $parent = Category::where('slug', $request->category)->first();
            if($parent){
                $res->where('parent_id', $parent->id);
            }
        }else{
            $res->whereNull('parent_id');
        }        

        $categories = $res->orderBy('position')->paginate($page);

        $categories->transform(function($category){
            $category->sstatus = $category->status == 'active';
            return $category;
        });        

        return response()->json([
            'status' => 'success',
            'categories' => $categories
        ]);
    }

    public function create(Request $request)
    {        
        $category = null;        
        
        $slug = $request->has('category') ? $request->category: '';
        $response = $this->pageRoute($slug);
        $route = $response['route'];
        
        $parent = Category::select('id', 'title', 'image')->whereNull('parent_id')->get();        
        $category = $response['category'];
        if(!is_null($category)){
            $parent = Category::select('id', 'title', 'image')->where('parent_id', $category->id)->get();
        }        
        
        $parent = Category::recursivescategories();        

        if($request->ajax()){
            return response()->json([
                'status' => 'success',
                "parent" => $parent, 
                "category" => $category,
                "settings" => settings()
            ]);
        }

        return view("catalogs::category.create", [            
            "route" => $route
        ]);
    }

    public function store(Request $request){
        try {
            DB::beginTransaction();            

            $status = json_decode($request->status);            
            $parent = json_decode($request->parent);            

            $category = Category::create([
                "title" => $request->title,
                "description" => $request->description,
                "code" => $request->code,
                "status" => $status->value,
                "parent_id" => $parent?->id ?? null,
                "background" => $request->background,
                "color" => $request->color,
                "parent_marge" => $request->marge,
                "created_at" => date_now(),
                "updated_at" => date_now(),
            ]);
            
            $slug = $category->id . '-' . generateString(12);
            $category->slug = $slug;
            $category->position = DB::raw('position + 1');

            if ($request->hasFile('file') && $request->file('file')->isValid()) {            
                $image = $request->file('file');
                $imageData = (new Images('images/categories'))->image_upload($image, [
                    'width' => 220,
                    'height' => 240,
                ]);
            
                $category->image = $imageData['filename'];
            }

            $category->save();  

            DB::commit();

            if($request->has('category')){
                $redirect = route('app.catalogs.category.edit', ['slug' => $slug, 'category' => $request->category]);
            }else{
                $redirect = route('app.catalogs.category.edit', ['slug' => $slug]);
            }

            return response()->json([
                'status' => 'success',
                'message' => message()['success'],
                'category' => [
                    'data' => Category::with('parent')->where('slug', $slug)->first(),
                    'url' => $redirect
                ]
            ]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }

    public function edit(String $slug, Request $request){                
        $response = $this->pageRoute($slug);
        $route = $response['route'];        

        if($request->ajax()){
            $parent = null;
            $parents = Category::select('id', 'title', 'image')->whereNull('parent_id')->get();
            $category = Category::where('slug', $slug)->with('parent')->first();

            if($request->has('category')){
                $parent = [
                    'id' => $category->parent->id,
                    'title' => $category->parent->title,
                    'image' => $category->parent->image,
                ];            
                $parents = Category::select('id', 'title', 'image')->where('id', $parent['id'])->get();                            
            }

            return response()->json([
                'status' => 'success',
                "parents" => $parents, 
                "parent" => $parent,
                "category" => $category
            ]);
        }
            
        return view('catalogs::category.edit', [            
            "route" => $route,            
        ]);
    }

    public function update(Request $request)
    {
        #dd($request->all());

        try {
            DB::beginTransaction();
            $id = $request->id;
            $status = json_decode($request->status);            
            $parent = json_decode($request->parent);            

            $parent_id = $parent?->id ?? null;
            Category::where('id', $id)->update([
                "title" => $request->title,
                "description" => $request->description,
                "code" => $request->code,
                "status" => $status->value,
                "parent_id" => $parent_id,
                "background" => $request->background,
                "color" => $request->color,
                "parent_marge" => $request->marge,       
                "updated_at" => date_now(),
            ]);

            if(is_null($parent_id)){
                Category::where('parent_id', $id)->update([
                    "parent_marge" => $request->marge,
                    "updated_at" => date_now(),
                ]);
            }

            if ($request->hasFile('file') && $request->file('file')->isValid()) {
                $image = $request->file('file');
                $imageData = (new Images('images/categories'))->image_upload($image, [
                    'width' => 220,
                    'height' => 240,
                ]);
                Category::where('id', $id)->update([
                    'image' => $imageData['filename']
                ]);                        
            }

            DB::commit();

            $category = Category::with('parent')->where('id', $id)->first();

            return response()->json([
                'status' => 'success',
                'message' => message()['update'],
                'category' => [
                    'data' => $category,
                    'url' => route('app.catalogs.category.edit', ['slug' => $category->slug])
                ]
            ]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }

    public function status(Request $request){         
        $id = $request->id;
        $status = !$request->status;
        
        $customer = Category::find($id);
        $customer->status = $status ? 'active' : 'inactive';
        $customer->save();

        return response()->json([
            'status' => 'success',
            'message' => message()['update'],            
        ]);
    }

    public function destroy($id)
    {        
        Category::find($id)->delete();
        Category::where("id", ">", $id)->update([
            "position" => DB::raw("position - 1")
        ]);
        
        return response()->json([
            'status' => 'success',
            'message' => 'Suppréssion éffectuée avec succès.',
        ]);
    }

    public function reorder(Request $request)
    {        
        $categories = $request->all();          
        $parent = $categories[0]['parent_id'];
        
        foreach($categories as $k => $item){
            $query = Category::query();
            if(!is_null($parent)){
                $query = Category::where('parent_id', $parent);
            }
            
            $query->where('id', $item['id'])->update([
                        'position' => $k + 1,
                        'updated_at' => date_now()
                    ]);
        }
        return response()->json([
            'status' => 'success',
            'message' => message()['update'],            
        ]);
    }
}
