<?php
namespace modules\sales\services;

use App\Models\BusinessContact;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use modules\crm\Models\BusinessContactTransaction;
use modules\crm\services\CrmService;
use modules\sales\Models\AccountingDocStatusLog;
use modules\sales\Models\AccountingDocumentLog;
use modules\sales\Models\AccountingDocumentPayment;
use modules\sales\Models\AccountingDocumentStatus;

class InvoiceService
{
    /**
     * Valide les données de la requête.
     */
    public function validatePaymentRequest(Request $request)
    {
        $request->validate([
            'amount' => 'required|numeric|not_in:0',
            'paid_date' => 'required|date',
            'method.id' => 'required'
        ]);
    }

    /**
     * Traite les paiements sur les documents.
     */
    public function processPayments($documents, Request $request, $user, $payment)
    {
        $reste = $request->amount;
        $referencePrefix = "PAIEMENT FACTURE VENTE ";

        foreach ($documents->sortBy('created_at') as $item) {
            /*$acompte = min($reste, abs($item->rap));
            $reste -= $acompte;*/                        
            #dd("$reste - ". abs($item->rap));
            $reste = $reste - abs($item->rap);
            $ispaid = $reste >= 0;
            
            $acompte = $ispaid ? abs($item->rap) : $reste;
            
            $this->createPaymentRecord($item, $user, $payment, $acompte, $request->additionnals, $request->observation);
            $this->createBusinessTransaction($item, $acompte, $payment, $referencePrefix);
            
            if ($reste == 0) {
                $this->updateDocumentStatus($item, $user);
                break;
            }

            $this->logAccountingDocument($item, $user, $referencePrefix);
        }
    }

    /**
     * Crée un enregistrement de paiement.
     */
    private function createPaymentRecord($item, $user, $payment, $amount, $additionals, $observation)
    {
        return AccountingDocumentPayment::create([
            "accounting_document_id" => $item->id,
            "reference" => $item->reference,
            "user_id" => $user->id,
            "username" => $user->name,
            "payment_method_id" => $payment['id'],
            "payment_method" => $payment['title'],
            "payment_additionnal" => json_encode($additionals),
            "amount" => $amount,
            "observation" => $observation,
            'created_at' => now(),
            'updated_at' => now(),
        ]);
    }

    /**
     * Crée une transaction commerciale.
     */
    private function createBusinessTransaction($item, $amount, $payment, $referencePrefix)
    {
        return BusinessContactTransaction::create([
            'business_contact_id' => $item->customer_id,
            'business_contact_name' => $item->customer,
            'document_id' => $item->id,
            'business_contact_type' => 'customer',
            'transaction_type' => 'in',
            'document_type' => 'invoice',
            'document_reference' => $item->reference,
            'document_amount' => $amount,
            'description' => $referencePrefix,
            'amount' => $amount,
            'payment_method' => json_encode($payment),
            'created_at' => now(),
            'updated_at' => now()
        ]);
    }

    /**
     * Met à jour le statut du document une fois payé.
     */
    private function updateDocumentStatus($item, $user)
    {
        $status = AccountingDocumentStatus::where('type', 'invoice')->where('pay', 1)->first();
        return AccountingDocStatusLog::create([
            'accounting_document_id' => $item->id,
            "reference" => $item->reference,
            'accounting_document_status_id' => $status->id,
            'accounting_document_status' => $status->title,
            'background' => $status->background,
            'color' => $status->color,
            'icon' => $status->icon,
            'user_id' => $user->id,
            "username" => $user->name,
            "created_at" => now(),
            "updated_at" => now(),
        ]);
    }

    /**
     * Enregistre un log du document comptable.
     */
    private function logAccountingDocument($item, $user, $referencePrefix)
    {
        $currentStatus = AccountingDocStatusLog::where('accounting_document_id', $item->id)->latest()->first();

        return AccountingDocumentLog::create([
            "accounting_document_id" => $item->id,
            "reference" => $item->reference,
            "accounting_document_status_id" => $currentStatus->accounting_document_status_id,
            "accounting_document_status" => $currentStatus->accounting_document_status,
            "title" => $referencePrefix,
            "user_id" => $user->id,
            "username" => $user->name,
            "created_at" => now(),
            "updated_at" => now(),
        ]);
    }

    /**
     * Génère la réponse de succès avec les transactions mises à jour.
     */
    public function generateSuccessResponse($customerId)
    {
        $businessContact = BusinessContact::find($customerId);
        $transactions = (new CrmService())->transactions($customerId, 25);
        $businessContact['transactions'] = $transactions;

        return response()->json([
            'status' => 'success',
            'business_contact' => $businessContact
        ]);
    }

}