<?php

namespace modules\stocks\Http\Controllers;

use App\Http\Controllers\Controller;
use GuzzleHttp\Psr7\Query;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use modules\catalogs\Models\ProductLocation;
use modules\catalogs\Models\ProductStock;
use modules\catalogs\Models\ProductSupplierPrice;
use modules\catalogs\services\ProductService;
use modules\catalogs\services\StockService;
use modules\location\Models\Location;
use modules\stocks\Models\StockTransfert;
use modules\stocks\Models\StockTransfertDetail;

class stocksController extends Controller
{
    public function index()
    {                
        return view('stocks::transfers.index');
    }

    
    public function detail(String $slug, Request $request)
    {
        if($request->ajax()){            
            $detail = StockTransfert::with(['details'])->where('slug', $slug)->first();
            $detail->details->transform(function($item){
                $item->product = ProductService::products(['id', $item->product_id])->first();
                return $item;
            });

            return response()->json([
                'status' => 'success',
                'detail' => $detail,
                'settings' => settings()
            ]);
        }
        return view('stocks::transfers.detail');
    }

    public function transferts(Request $request)
    {
        $page = isset($request->pagesize) ? $request->pagesize : 25;
        $query = StockTransfert::with(['details']);

        if ($request->has('criteria')) {
            if ($request->criteria == 'source') {
                $query->where('source_location', 'LIKE', '%'.$request->keyword['text'].'%');
            }
            if ($request->criteria == 'date') {
                $query->whereDate('requested_at', $request->keyword);
            }
            if ($request->criteria == 'destination') {
                $query->where('destination_location', 'LIKE', "%".$request->keyword['text']."%");
            }
            if ($request->criteria == 'status') {                
                $query->where('status', $request->keyword['value']);
            }
        }

        $location = my_location();
        if($location){
            $query->where('source_location_id', $location->id)
                    ->orWhere('destination_location_id', $location->id);
        }

        $query->orderBy('requested_at');
        if((int)$page == 0){
            $transferts = $query->get();
        }else{
            $transferts = $query->paginate($page);
        }

        $permissions = array_values(auth()->user()->getAllPermissions()->filter(function ($permission) {
            return str_contains($permission->name, 'transfert');
        })->toArray());

        if(is_null(Auth::user()->location_id)){
            $locations = Location::where('status', 'active')->where('stock_management', 1)->get();
        }else{
            $locations = Location::where('id', Auth::user()->location_id)->where('stock_management', 1)->get();
        }

        return response()->json([
            'status' => 'success',
            'transferts' => $transferts,
            'message' => 'Success',
            'permissions' => $permissions,
            'locations' => $locations
        ]);
    }

    public function statusTransefrt(Request $request)
    {
        try {
            $transfert = StockTransfert::find($request->id);                
            $data['status'] = $request->status['name'];

            if($request->status['name'] == "approved"){
                DB::beginTransaction();
                
                $data['approved_at'] = date_now();
                $data['approved_by'] = auth()->id();
                $data['approved_username'] = auth()->user()->name;
                
                StockTransfert::where('slug', $request->slug)->update($data);
                $details = StockTransfertDetail::where('stock_transfer_id', $transfert->id)->get()->toArray();                
                $this->applyProrata($transfert, $details);

                DB::commit();
            }
            
            return response()->json([
                'status' => 'success'
            ]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }

    public function storeTransfert(Request $request)
    {
        try {
            #dd('store');
            DB::beginTransaction();            
            
            $data['notes'] = $request->observation;
            $data['updated_at'] = date_now();            
            StockTransfert::where('slug', $request->slug)->update($data);

            $details = $request->details;
            #dd($details);
            foreach($details as $detail){
                StockTransfertDetail::where('id', $detail['id'])->update([
                    "transferred_quantity" => $detail['transferred_quantity'],
                    "updated_at" => date_now()
                ]);
            }

            DB::commit();
            return response()->json([
                'status' => 'success'
            ]);
        } catch (\Throwable $th) {
            DB::rollBack();
            throw $th;
        }
    }

    private function applyProrata(StockTransfert $transfert, Array $details)
    {        
        $deliveryCost = $transfert->delivery_price;

        // Calcul de la valeur totale des produits reçus
        $detailsCollection = collect($details);        

        // Calcul de la valeur totale des produits reçus
        $totalValue = $detailsCollection->sum(function ($detail) {
            $qty = $detail['transferred_quantity'];
            return $qty * $detail['price_ht']; // Accéder aux éléments du tableau
        });

        if ($totalValue <= 0) {
            throw new \Exception("Valeur totale des produits reçus invalide.");
        }        
                    
        $source = Location::findOrfail($transfert->source_location_id);
        $destination = Location::findOrfail($transfert->destination_location_id);
        $destination_id = $destination->id;
        $source_id = $source->id;

        // Appliquer le calcul du prorata à chaque produit
        foreach ($details as $detail) {
            $transferedqty = $detail['transferred_quantity'];
            $productValue = $transferedqty * $detail['price_ht'];
            $productShare = $this->calculateProrata($productValue, $totalValue, $deliveryCost);
            $newCost = $detail['price_ht'] + ($productShare / $transferedqty);
        
            $product_id = $detail['product_id'];
            
            // Mise à jour du prix du produit à l'emplacement de destination
            ProductLocation::where(['product_id' => $product_id, 'location_id' => $destination_id])
                ->update([
                    'purchase_price' => $detail['price_ht'],
                    'cost_price' => $newCost,
                    'frns_price' => $newCost,
                    'frns_id' => $destination_id
                ]);
        
            // Vérification et insertion du prix fournisseur si nécessaire
            $existingPrice = ProductSupplierPrice::where([
                'product_id' => $product_id,
                'supplier_id' => $source_id,
                'location_id' => $destination_id
            ])->latest()->value('price');
        
            if (!$existingPrice || $existingPrice != $newCost) {
                ProductSupplierPrice::create([
                    'product_id' => $product_id,
                    'supplier_id' => $source_id,
                    'location_id' => $destination_id,
                    'price' => $newCost,
                    'created_at' => now(),
                    'updated_at' => now(),
                ]);
            }
        
            // Mise à jour des stocks pour la destination et la source
            $stock = new StockService();
            $transfertTitle = "transfert de stock de la facture " . $transfert->document;        
            $this->handleStockUpdate($stock, $product_id, $destination_id, $transferedqty, $newCost, "APPRO: " . $transfertTitle);            
        }    
    }

    private function handleStockUpdate($stock, $product_id, $location_id, $quantity, $price, $title) {
        $product = ProductService::products(['product_id' => $product_id, 'location' => $location_id])->first();
        $prodStock = ProductStock::where([
            'product_id' => $product_id, 
            'location_id' => $location_id
        ])->first();
        
        $stock_actual = $prodStock->stock;
        $stock_final = $stock_actual + $quantity;        
        $stock->update([
            'product_id' => $product_id,
            'location_id' => $location_id,
            'quantity' => abs($quantity),
            'price' => $price
        ], $quantity > 0 ? 'entry' : 'exit');
    
        $stock->createUpdateControl([
            'type' => $quantity > 0 ? 'in' : 'out',
            'where' => ['product_id' => $product_id, 'location_id' => $location_id],
            'location_id' => $location_id,
            'product_id' => $product_id,
            'product_name' => $product->title,
            'stock_actual' => $stock_actual,
            'quantity' => abs($quantity),
            'stock_final' => $stock_final,
            'title' => $title
        ]);
    
        $stock->logStockStory([
            'product_id' => $product_id,
            'product_name' => $product->title,
            'location_id' => $location_id,
            'title' => $title,
            'stock_actual' => $stock_actual,
            'quantity' => abs($quantity),
            'stock_final' => $stock_final,
            'token' => generateString(18),
        ]);
    }
    
    private function calculateProrata($productValue, $totalValue, $deliveryCost)
    {        
        return ($productValue / $totalValue) * $deliveryCost;
    }




}